# sample_eleven.py

"""

Author(s)  : ???
             Modified and updated for wxPython Phoenix by Ecco
Version    : 0.0.2
Website    : Python-it.org

"""

import os
import sys
import wx
import wx.lib.mixins.listctrl as listmix

# class MyDataSource
# class MyDefaultItemAttr
# class MyProperty
# class MyListCtrl
# class MyFrame
# class MyApp

#---------------------------------------------------------------------------

class MyDataSource(object):
    """
    Questa classe acquisiscie i dati da visualizzare nel wx.ListCtrl Virtuale.
    Per il suo corretto funzionamento i dati devono provenire da :
        - una lista per le intestazioni delle colonne;
        - una lista di liste, per le righe.
    I parametri passati all'__init__ sono :
        col per le intestazioni delle colonne;
        row per le righe.
        
    #------------
            
    Cette classe acquiere les donnees a afficher dans la wx.ListCtrl virtuelle.
    Pour un fonctionnement correct, les donnees doivent provenir de :
        - Une liste pour les en-tetes de colonnes;
        - Une liste des listes, pour les lignes.
    Les parametres passes a __init__ sont :
        col pour les en-tetes de colonnes;
        row pour les lignes.
    """

    def __init__(self, col, row):
        """
        ...
        """
        
        self.col = col
        self.row = row

    #-----------------------------------------------------------------------
        
    def GetColumnHeaders(self):
        """
        Ritorna la lista delle Colonne.
        Retourne la liste des colonnes.
        """

        return self.col


    def GetCount(self):
        """
        Ritorna il numero delle Righe.
        Retourne le nombre de rangees.
        """
        
        return len(self.row)


    def GetColCount(self):
        """
        Ritorna il numero delle Colonne.
        Retourne le nombre de colonnes.
        """
        
        return len(self.col)


    def GetItem(self, index):
        """
        Ritorna il valore della cella passato ad index.
        Retourne la valeur de la cellule transmise a l'index.
        """
        
        return self.row[index]


    def UpdateCache(self, start, end):
        pass
    
#---------------------------------------------------------------------------
    
class MyDefaultItemAttr(object):
    """
    Questa e' una classe di comodo per l'assegnazione dei valori di default
    dei tre attributi Font, BackgroundColour, TextColour.

    #------------
    
    C'est une  classe de commodite pour l'attribution des valeurs par defaut
    de ces trois attributs Font, BackgroundColour, TextColour.
    """

    default_font = (8, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL)
    default_BackgroundColour = "white"
    default_TextColour = "black"

    default_itemAttr = [default_font, default_BackgroundColour, default_TextColour]

#---------------------------------------------------------------------------
        
class MyProperty(object):
    """
    Questa classe fornisce le varie proprieta' al wx.ListCtrl Virtuale :
        width e' una lista di valori per la larghezza delle colonne, valore di Default ==> [-1];
        align e' una lista di valori per l'allinemento delle colonne, valore di Default ==> [wx.LIST_FORMAT_LEFT];
        itemAttr e' una lista di valori per gli Attributi di Riga,
            permette la creazione di tanti wx.ItemAttr per ogni elemente presente nella lista,
            ogni singolo elemento e' composto a sua volta da tre valori, rispettivamente per il Font, per il BackgroundColour e il TextColour
            valore di Default ==> [(8, wx.FONTFAMILY_DEFAULT  , wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL), "white", "black"]

        L'__init__ prende un parametro :
            property che e' un dizionario con le proprieta' su elencate i cui indici sono le proprieta' stesse.

    #------------
    
     Cette classe fournie diverses proprietes a la wx.ListCtrl virtuelle :
        width est une liste de valeurs pour la largeur des colonnes, valeur par defaut ==> [-1];
        align est une liste de valeurs pour l'alignement des colonnes, valeur par defaut, ==> [wx.LIST_FORMAT_LEFT];
        itemAttr et une liste de valeurs pour les attributs de Riga,
            permet la creation de nombreux wx.ItemAttr pour chaque element present dans la liste,
            chaque element individuel est constitue a son tour par trois valeurs, respectivement pour Font, pour BackgroundColour et TextColour
            La valeur par defaut ==> [(8, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL), "white", "black"]

        La methode __ init__ prend un parametre :
            la propriete est un dictionnaire avec les proprietes dont les indices ont des proprietes qui leur sont propre.
    """
    
    def __init__(self, property):
        """
        ...
        """
        
        self.property = property
        
        self.default_font = MyDefaultItemAttr().default_itemAttr
        self.default_BackgroundColour = MyDefaultItemAttr().default_BackgroundColour
        self.default_TextColour = MyDefaultItemAttr().default_TextColour
        self.default_itemAttr = MyDefaultItemAttr().default_itemAttr

    #-----------------------------------------------------------------------
        
    def get_property(self):
        """
        Restituisce il dizionario con le propriea' del wx.ListCtrl Virtuale.    
        Retourne le dictionnaire avec les proprietes de la wx.ListCtrl virtuelle.
        """

        return {
                "width": self._get_width(),
                "align": self._get_align(),
                "itemAttr": self._get_itemAttr()
               }


    def _get_property(self, index, default):
        """
        Metodo interno di comodo per il vari get_*.
        Se index non e' presente nel dizionario fa ritornare il valore di Default.

        Methode interne de commodite pour les differents get_*.
        Si l'index n'est pas present dans le dictionnaire il retourne la valeur par defaut.
        """
        
        if not self.property.get(index, False):
            return default
        else:
            return self.property[index]


    #--- Metodi per assegnare i valori alle proprieta'.
    # --- Methodes pour attribuer des valeurs a la propriete.
    def _get_width(self):
        """
        Ritorna il valore di width.
        Retourne la valeur de la largeur.
        """

        return self._get_property("width", [-1])


    def _get_align(self):
        """
        Ritorna il valore di align.
        Retourne la valeur de l'alignement.
        """

        return self._get_property("align", [wx.LIST_FORMAT_LEFT])

    
    def _get_itemAttr(self):
        """
        Ritorna una lista di wx.ItemAttr per Font, Colore Sfondo e Colore Testo
        Se uno o piu' valori non sono presenti assegna quello di default.

        Retourne une liste des wx.ItemAttr pour la Font, le BackgroundColour et le ForegroundColour du texte
        Si une ou plusieurs valeurs ne sont pas presentes il attribue la valeur par defaut.
        """
        
        def _assign_value(list1, list2):
            attr = wx.ItemAttr()
            try: attr.SetFont(wx.Font(*list1[0]))
            except IndexError and TypeError: attr.SetFont(wx.Font(*list2[0]))
            
            try: attr.SetBackgroundColour(list1[1])
            except IndexError: attr.SetBackgroundColour(list2[1])

            try: attr.SetTextColour(list1[2])
            except IndexError: attr.SetTextColour(list2[2])

            return attr
        
        list_attr = []
        for l in self.property.get("itemAttr", [self.default_itemAttr]):
            list_attr.append(_assign_value(l, self.default_itemAttr))

        return list_attr

#---------------------------------------------------------------------------
    
class MyListCtrl(wx.ListCtrl,
                 listmix.ColumnSorterMixin,
                 listmix.ListCtrlAutoWidthMixin):
    """
    Questa classe e' il cuore pulsante di MyListCtrl.
    L'__init__ richiede due parametri:
        dataSource che e' la lista dei valori da visualizzare nella lista;
        property che e' il dizionario delle proprieta' della lista, questo parametro e' facoltativo.

    #------------
    
    Cette classe et le cœur de MyListCtrl.
    La methode __ init__ requiert deux parametres:
        dataSource est la liste des valeurs a afficher dans la liste;
        property et le dictionnaire de la liste des proprietes, ce parametre est optionnel.
    """
    
    def __init__(self, *args, **kwargs):
        
        self.dataSource = kwargs.pop("dataSource")

        #--- Se property non viene dichiarato ne viene ritornato uno vuoto.
        #--- Si la propriete n'est pas declaree, elle est renvoyee a vide.
        try:property = kwargs.pop("property")
        except KeyError: property=MyProperty({})

        #---
        
        wx.ListCtrl.__init__(self,
                             style=wx.LC_REPORT |
                                   wx.LC_VIRTUAL |
                                   wx.LC_SINGLE_SEL |
                                   wx.LC_VRULES |
                                   wx.LC_HRULES,
                             *args, **kwargs)

        #------------

        # Initialize the listCtrl auto width.
        listmix.ListCtrlAutoWidthMixin.__init__(self)

        #------------

        # Return bitmaps folder.
        self.bitmaps_dir = wx.GetApp().GetBitmapsDir()
        # Return icons folder.
        self.icons_dir = wx.GetApp().GetIconsDir()

        #------------

        # Load some images into an image list.
        self.il = wx.ImageList(16, 16, True)

        # Add arrows for the column sorter.
        self.bmp1 = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                           "sm_up.png"))
        self.bmp2 = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                           "sm_down.png"))
        
        # Add image for rows.
        self.bmp3 = wx.Bitmap(os.path.join(self.bitmaps_dir,
                                               "folder.png"))
        
        # Add images to list.
        self.sm_up = self.il.Add(self.bmp1)
        self.sm_dn = self.il.Add(self.bmp2)
        self.idx1 = self.il.Add(self.bmp3)

        empty = self.makeBlank()
        self.idx2 = self.il.Add(empty)
        
        # Assign the image list to it.
        self.SetImageList(self.il, wx.IMAGE_LIST_SMALL)

        #------------
        
        self.oldsortcolumn = -1 
        self.sort_asc = False
        
        #------------
        
        width = property.get_property()["width"]            # Larghezza colonne.       # Largeur de colonne.
        align = property.get_property()["align"]            # Allineamento colonne.    # Alignement de colonne.
        self.itemAttr = property.get_property()["itemAttr"] # Attributi righe.         # Attributs de lignes. 

        #--- Crea le colonne :
        #--- Creation de colonnes :
        #    Se le liste non sono complete per tutte le colonne passa i parametri alle prime n colonne.
        #    per le restanti passa i valori di Default, wx.LIST_FORMAT_LEFT per align e -1 per width.

        # Si les listes ne sont pas completes pour toutes les colonnes transmet les parametres aux premieres colonnes.
        # Pour le reste, passe les valeurs par defaut, wx.LIST_FORMAT_LEFT pour aligner et -1 pour la largeur.
        columns = self.dataSource.GetColumnHeaders()
        for col, text in enumerate(columns):
            try: Width = width[col]
            except IndexError: Width = -1

            try: Align = align[col]
            except IndexError: Align = wx.LIST_FORMAT_LEFT

            self.InsertColumn(col, text, Align, Width)
 
        #--- mixin 
        data = dict(enumerate(self.dataSource.row))
        print("Data dict :", data)
        self.itemDataMap = data
        print("Item data map :", self.itemDataMap)
        self.itemIndexMap = data.keys()
        print("Item index map :", self.itemIndexMap)

        #------------

        # Initialize the column sorter.
        listmix.ColumnSorterMixin.__init__(self, self.GetColumnCount())

        #------------
      
        self.SetItemCount(len(data))
        # self.SortListItems(0, True)
        
    #-----------------------------------------------------------------------

    def makeBlank(self):
        """
        ...
        """
        
        empty = wx.Bitmap(16, 16, 32)
        dc = wx.MemoryDC(empty)
        dc.SetBackground(wx.Brush((0, 0 ,0, 0)))
        dc.Clear()
        del dc
        empty.SetMaskColour((0, 0, 0))
        return empty


    #--- Metodi richiamati per la virtualita' della lista.
    #--- Methodes appelees pour la liste virtuelle.
    def OnGetItemText(self, item, col):
        """
        ...
        """
        
        # index=self.itemIndexMap[item]
        # return self.itemDataMap[index][col]
        
        data = self.dataSource.GetItem(item)
        return str(data[col])


    def OnGetItemImage(self, item):
        """
        Return item image (folder).
        """

        return self.idx1


    def OnGetItemAttr(self, item):
        """
        ...
        """
        
        return self.itemAttr[item % len(self.itemAttr)]

    
    #--- Metodi richiamati per mixin.
    #--- Methodes appelees pour mixin.
    def GetListCtrl(self):
        """
        ...
        """
        
        return self

    
    # Used by the ColumnSorterMixin,
    # see wx/lib/mixins/listctrl.py
    def GetSortImages(self):        # Not used.
        """
        Return sort images.
        """

        return (self.sm_dn, self.sm_up)

    
    def OnSortOrderChanged(self):
        """
        It's better but is it correct ?
        """
        
        data = self.dataSource.row

        #------------

        if self.oldsortcolumn != -1:
            self.sort_asc = True
            self.ClearColumnImage(self.oldsortcolumn)
        else:
            if self.sort_asc:
                self.sort_asc = None
            else:
                self.sort_asc = True

        if self.sort_asc:
            if self._colSortFlag[self._col]:
                data.sort(key=lambda x: x[self._col])
                idx = self.sm_up
            else:
                data.sort(key=lambda x: x[self._col])
                data.reverse()
                idx = self.sm_dn
            
        #------------
                
        self.SetColumnImage(self._col, idx)

        #------------
        
        self.oldsortcolumn = self._col

        #------------
        
        self.DeleteAllItems()

        #------------
        
        self.SetItemCount(len(data))
        

    # def SortItems(self, sorter=cmp):
        # items = list(self.itemDataMap.keys())
        # items.sort(sorter)
        # self.itemIndexMap = items
        # ridisegna la lista.
        # Redessine la liste.
        # self.Refresh()    

#---------------------------------------------------------------------------
        
class MyFrame(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, -1,
                          "Sample eleven (properties & row/header icon)",
                          size=(500, 506))
        
        #------------
        
        self.SetIcon(wx.Icon('icons/wxwin.ico'))
        self.SetMinSize((500, 506))

        #------------
        
        panel = wx.Panel(self, -1)

        #------------

        # Firstname, Sex, Age     
        col = ["Nome", "Sesso", "Eta"]

        #------------
        
        row = [
               ["Antonio", "M", 36],
               ["Letizia", "F", 39],
               ["Antonio", "M", 35],
               ["Letizia", "F", 39],
               ["Roberta", "F", 7],
               ["Francesca", "F", 61],
              ]*10   # Nombre de rangees.

        #------------
        
        data = MyDataSource(col, row)

        #------------
        
        property = {"align":[wx.LIST_FORMAT_LEFT, wx.LIST_FORMAT_CENTRE, wx.LIST_FORMAT_RIGHT],
                    "width":[150]*3,
                    "itemAttr": [
                                 [(10, wx.FONTFAMILY_ROMAN, wx.NORMAL, wx.NORMAL), "white", "black"],
                                 [(10, wx.FONTFAMILY_ROMAN, wx.NORMAL, wx.BOLD), (90, 100, 50), (150, 150, 150)],
                                 # MyDefaultItemAttr().default_itemAttr
                                ]
                    }

        #------------
        
        data_property = MyProperty(property)
        
        self.list1 = MyListCtrl(panel, dataSource=data)
        self.list2 = MyListCtrl(panel, dataSource=data, property=data_property)
        
        #------------
        
        box_sizer = wx.BoxSizer(wx.VERTICAL)

        # MyListCtrl sans proprietes.       
        box_sizer.Add(wx.StaticText(panel, -1, "MyListCtrl senza 'property'"))  
        box_sizer.Add(self.list1, 1, wx.EXPAND|wx.BOTTOM, 15)
        # MyListCtrl avec proprietes.
        box_sizer.Add(wx.StaticText(panel, -1, "MyListCtrl con 'property'"))  
        box_sizer.Add(self.list2, 1, wx.EXPAND)

        panel.SetSizer(box_sizer)
        
#---------------------------------------------------------------------------

class MyApp(wx.App):    
    def OnInit(self):

        #------------

        self.installDir = os.path.split(os.path.abspath(sys.argv[0]))[0]
        
        #------------

        frame = MyFrame()
        self.SetTopWindow(frame)
        frame.Show(True)

        return True

    #-----------------------------------------------------------------------
    
    def GetInstallDir(self):
        """
        Return the installation directory for my application.
        """

        return self.installDir


    def GetIconsDir(self):
        """
        Return the icons directory for my application.
        """

        icons_dir = os.path.join(self.installDir, "icons")
        return icons_dir


    def GetBitmapsDir(self):
        """
        Return the bitmaps directory for my application.
        """

        bitmaps_dir = os.path.join(self.installDir, "bitmaps")
        return bitmaps_dir
    
#---------------------------------------------------------------------------

def main():
    app = MyApp(False)
    app.MainLoop()

#---------------------------------------------------------------------------

if __name__ == "__main__" :
    main()
