# sample_five.py

# http://bty.sakura.ne.jp/wp/archives/52

import wx
import wx.lib.mixins.listctrl as listmix

# class MyFrame
# class MyListCtrl
# class MyApp

#---------------------------------------------------------------------------

caption = ("ID", "Name", "X", "Y", "Z")

#---------------------------------------------------------------------------

data = [
    (1, "Noel", 5570, 2638, 2933),
    (2, "Martin", 1407, 663, 744),
    (3, "Antoine", 1364, 652, 712),
    (4, "Jenifer", 2347, 1140, 1208),
    (5, "Marc", 1121, 527, 593),
    (6, "Etienne", 1198, 575, 623),
    (7, "Louise", 2067, 1004, 1063),
    (8, "Justine", 2969, 1477, 1492),
    (9, "Eloise", 2014, 1001, 1013),
    (10, "Marguerite", 2016, 993, 1024),
    (11, "Sophie", 7090, 3570, 3520),
    (12, "Edith", 6098, 3047, 3051),
    (13, "Jacob", 12758, 6354, 6405),
    (14, "Nelly", 8880, 4484, 4396),
    ]

#---------------------------------------------------------------------------
        
class MyFrame(wx.Frame):
    def __init__(self, parent, id):
        wx.Frame.__init__(self, parent, id,
                          "Sample five (copy to wx.TheClipboard & sort list)",
                          size=(450, 300))

        #------------
        
        self.SetIcon(wx.Icon('icons/wxwin.ico'))
        self.SetMinSize((450, 300))
        
        #------------

        self.panel = wx.Panel(self, -1)
        self.btnCopy = wx.Button(self.panel, -1, "Copy")
        self.Bind(wx.EVT_BUTTON, self.OnBtnCopy, self.btnCopy)
        
        self.listCtrl = MyListCtrl(self.panel)

        #------------
        
        mainSizer = wx.BoxSizer(wx.VERTICAL)
        btnSizer = wx.BoxSizer(wx.HORIZONTAL)
        
        btnSizer.Add(self.btnCopy, 1)
        
        mainSizer.Add(btnSizer, 0, wx.EXPAND)
        mainSizer.Add(self.listCtrl, 1, wx.EXPAND)

        self.panel.SetSizer(mainSizer)
        
    #-----------------------------------------------------------------------
        
    def OnBtnCopy(self, event):
        """
        ...
        """
        
        if not wx.TheClipboard.Open():
            wx.MessageBox("Unable to copy to clipboard !")
            return

        text = ""
        for i in range(self.listCtrl.GetItemCount()):
            item = self.listCtrl.GetItem(i)
            if item.GetState() & wx.LIST_STATE_SELECTED:
                d = self.listCtrl.items[i]
                text += str(d[0])+' '+d[1]+' '+str(d[2])+' '+str(d[3])+' '+str(d[4])+'\r\n'
                print("Copy to wx.TheClipboard :", text)
                
        do = wx.TextDataObject(text)
        wx.TheClipboard.SetData(do)
        wx.TheClipboard.Close()
        
#---------------------------------------------------------------------------

class MyListCtrl(wx.ListCtrl,
                 listmix.ListCtrlAutoWidthMixin):
    def __init__(self, parent):
        wx.ListCtrl.__init__(self, parent, -1,
                             style=wx.LC_REPORT |
                                   wx.LC_VIRTUAL |
                                   wx.LC_HRULES |
                                   wx.LC_VRULES)

        #------------

        # Initialize the listCtrl auto width.
        listmix.ListCtrlAutoWidthMixin.__init__(self)

        #------------

        self.prevColumn = False
        self.sortAcend = True

        #------------
        
        self.SetBackgroundColour("#ffbb55")
        
        #------------

        self.items = data
        print("Items :",  self.items)

        #------------
        
        self.InsertColumn(0, caption[0], wx.LIST_FORMAT_RIGHT)
        self.InsertColumn(1, caption[1], wx.LIST_FORMAT_LEFT)
        self.InsertColumn(2, caption[2], wx.LIST_FORMAT_RIGHT)
        self.InsertColumn(3, caption[3], wx.LIST_FORMAT_RIGHT)
        self.InsertColumn(4, caption[4], wx.LIST_FORMAT_RIGHT)

        #------------
        
        self.SetItemCount(len(self.items))
        
        #------------
        
        self.Bind(wx.EVT_LIST_COL_CLICK, self.Sort)
 
    #-----------------------------------------------------------------------

    def GetListCtrl(self):
        """
        ...
        """     
        
        return self

    
    def OnGetItemText(self, item, col):
        """
        ...
        """
        
        items = (self.items[item][col])
        
        return str(items)

    
    def Sort(self, event):
        """
        ...
        """
        
        col = event.GetColumn()

        #------------
        
        if col != self.prevColumn:
            self.sortAcend = True
        else:
            if self.sortAcend:
                self.sortAcend = None
            else:
                self.sortAcend = True
        if self.sortAcend:
            self.items.sort(key=lambda x: x[col])
        else:
            self.items.reverse()

        self.prevColumn = col

        #------------
        
        self.DeleteAllItems()

        #------------
        
        self.SetItemCount(len(self.items))

#---------------------------------------------------------------------------

class MyApp(wx.App):    
    def OnInit(self):

        #------------

        frame = MyFrame(parent=None, id=-1)
        self.SetTopWindow(frame)
        frame.Show(True)

        return True
    
#---------------------------------------------------------------------------

def main():
    app = MyApp(False)
    app.MainLoop()

#---------------------------------------------------------------------------

if __name__ == "__main__" :
    main()
