# sample_seven.py

import wx
import wx.lib.mixins.listctrl as listmix

# class MyFrame
# class MyListCtrl
# class MyApp

#---------------------------------------------------------------------------

caption = ("ID", "Name", "X", "Y", "Z")

#---------------------------------------------------------------------------

data = [
    (1, "Noel", 5570, 2638, 2933),
    (2, "Martin", 1407, 663, 744),
    (3, "Antoine", 1364, 652, 712),
    (4, "Jenifer", 2347, 1140, 1208),
    (5, "Marc", 1121, 527, 593),
    (6, "Etienne", 1198, 575, 623),
    (7, "Louise", 2067, 1004, 1063),
    (8, "Justine", 2969, 1477, 1492),
    (9, "Eloise", 2014, 1001, 1013),
    (10, "Marguerite", 2016, 993, 1024),
    (11, "Sophie", 7090, 3570, 3520),
    (12, "Edith", 6098, 3047, 3051),
    (13, "Jacob", 12758, 6354, 6405),
    (14, "Nelly", 8880, 4484, 4396),
    ]

#---------------------------------------------------------------------------
        
class MyFrame(wx.Frame):
    def __init__(self, parent, id):
        wx.Frame.__init__(self, parent, id,
                          "Sample seven (popup menu)",
                          size=(450, 295))

        #------------
        
        self.SetIcon(wx.Icon('icons/wxwin.ico'))
        self.SetMinSize((450, 295))

        #------------
        
        self.listCtrl = MyListCtrl(self)
        
#---------------------------------------------------------------------------

class MyListCtrl(wx.ListCtrl,
                 listmix.ListCtrlAutoWidthMixin):
    def __init__(self, parent):
        wx.ListCtrl.__init__(self, parent, -1,
                             style=wx.LC_REPORT |
                                   wx.LC_VIRTUAL |
                                   wx.LC_HRULES |
                                   wx.LC_VRULES)

        #------------

        # Initialize the listCtrl auto width.
        listmix.ListCtrlAutoWidthMixin.__init__(self)

        #------------

        self.prevColumn = False
        self.sortAcend = True

        #------------
        
        self.SetBackgroundColour("#8be587")
        
        #------------

        self.items = data
        print("Items :",  self.items)

        #------------
        
        self.InsertColumn(0, caption[0], wx.LIST_FORMAT_RIGHT)
        self.InsertColumn(1, caption[1], wx.LIST_FORMAT_LEFT)
        self.InsertColumn(2, caption[2], wx.LIST_FORMAT_RIGHT)
        self.InsertColumn(3, caption[3], wx.LIST_FORMAT_RIGHT)
        self.InsertColumn(4, caption[4], wx.LIST_FORMAT_RIGHT)

        #------------
        
        self.SetItemCount(len(self.items))
        
        #------------
        
        self.Bind(wx.EVT_LIST_COL_CLICK, self.Sort)
        
        #------------
        
        # For wxMSW.
        self.Bind(wx.EVT_COMMAND_RIGHT_CLICK, self.OnRightClick)

        # For wxGTK.
        self.Bind(wx.EVT_RIGHT_UP, self.OnRightClick)
        
    #-----------------------------------------------------------------------

    def OnRightClick(self, event):
        """
        ...
        """

        # Only do this part the first time 
        # so the events are only bound once.
        if not hasattr(self, "popupID1"):
            self.popupID1 = wx.NewIdRef()
            self.popupID2 = wx.NewIdRef()
            self.popupID3 = wx.NewIdRef()

            self.Bind(wx.EVT_MENU, self.OnPopupOne, id=self.popupID1)
            self.Bind(wx.EVT_MENU, self.OnPopupTwo, id=self.popupID2)
            self.Bind(wx.EVT_MENU, self.OnPopupThree, id=self.popupID3)

        #------------
            
        # Make a menu.
        menu = wx.Menu()
        
        # Add some items.
        menu.Append(self.popupID1, "First message box")
        menu.Append(self.popupID2, "Second message box")
        menu.Append(self.popupID3, "Delete all items")

        # Popup the menu.  If an item is selected then its 
        # handler will be called before PopupMenu returns.
        self.PopupMenu(menu)
        menu.Destroy()


    def OnPopupOne(self, event):
        """
        ...
        """
        
        dlg = wx.MessageDialog(self,
                               "Hello from Python and wxPython !",
                               "First message box",
                               wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()
        

    def OnPopupTwo(self, event):
        """
        ...
        """
        
        dlg = wx.MessageDialog(self,
                               "Welcome !",
                               "Second message box",
                               wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()


    def OnPopupThree(self, event):
        """
        ...
        """
        
        self.DeleteAllItems()

        
    def GetListCtrl(self):
        """
        ...
        """     
        
        return self

    
    def OnGetItemText(self, item, col):
        """
        ...
        """
        
        items = (self.items[item][col])
        
        return str(items)

    
    def Sort(self, event):
        """
        ...
        """
        
        col = event.GetColumn()

        #------------
        
        if col != self.prevColumn:
            self.sortAcend = True
        else:
            if self.sortAcend:
                self.sortAcend = None
            else:
                self.sortAcend = True
        if self.sortAcend:
            self.items.sort(key=lambda x: x[col])
        else:
            self.items.reverse()

        self.prevColumn = col

        #------------
        
        self.DeleteAllItems()

        #------------
        
        self.SetItemCount(len(self.items))

#---------------------------------------------------------------------------

class MyApp(wx.App):    
    def OnInit(self):

        #------------

        frame = MyFrame(parent=None, id=-1)
        self.SetTopWindow(frame)
        frame.Show(True)

        return True
    
#---------------------------------------------------------------------------

def main():
    app = MyApp(False)
    app.MainLoop()

#---------------------------------------------------------------------------

if __name__ == "__main__" :
    main()
